#!/usr/bin/env python

import unittest

import _otf2


def traverse(local_id, global_id, user_data):
    user_data[global_id] = local_id


class TestOTF2AnchorFile(unittest.TestCase):
    def test_1(self):
        id_map = _otf2.IdMap_Create(_otf2.ID_MAP_SPARSE, 10)
        self.assertIsNotNone(id_map)
        _otf2.IdMap_Free(id_map)

    def test_2(self):
        id_map = _otf2.IdMap_Create(_otf2.ID_MAP_SPARSE, 100)
        self.assertIsNotNone(id_map)
        _otf2.IdMap_Free(id_map)

    def test_3(self):
        id_map = _otf2.IdMap_Create(_otf2.ID_MAP_SPARSE, 6)
        self.assertIsNotNone(id_map)

        _otf2.IdMap_AddIdPair(id_map, 42, 0)
        _otf2.IdMap_AddIdPair(id_map, 13, 1)
        _otf2.IdMap_AddIdPair(id_map, 0, 2)

        array = [0, 0, 0]
        _otf2.IdMap_Traverse(id_map, traverse, array)

        self.assertEqual(42, array[0])
        self.assertEqual(13, array[1])
        self.assertEqual(0, array[2])

        _otf2.IdMap_Free(id_map)

    def test_4(self):
        mappings = [0, 5, 2, 1, 4]

        id_map = _otf2.IdMap_CreateFromUint64Array(mappings, True)
        self.assertIsNotNone(id_map)

        mode = _otf2.IdMap_GetMode(id_map)
        self.assertEqual(mode, _otf2.ID_MAP_SPARSE)

        for i in range(5):
            global_id = _otf2.IdMap_GetGlobalId(id_map, i)
            self.assertEqual(mappings[i], global_id)
        _otf2.IdMap_Free(id_map)

    def test_5(self):
        mappings = [0, 5, 2, 1, 3]

        id_map = _otf2.IdMap_CreateFromUint64Array(mappings, True)
        self.assertIsNotNone(id_map)

        mode = _otf2.IdMap_GetMode(id_map)
        self.assertEqual(mode, _otf2.ID_MAP_DENSE)

        for i in range(5):
            global_id = _otf2.IdMap_GetGlobalId(id_map, i)
            self.assertEqual(mappings[i], global_id)

        _otf2.IdMap_Free(id_map)

    def test_6(self):
        mappings = [0, 1, 2, 3, 4]

        id_map = _otf2.IdMap_CreateFromUint64Array(mappings, True)
        self.assertEqual(id_map, None)

        id_map = _otf2.IdMap_CreateFromUint64Array([], True)
        self.assertEqual(id_map, None)

    def test_7(self):
        ids = [4, 0, 2, 1, 3]

        id_map = _otf2.IdMap_Create(_otf2.ID_MAP_SPARSE, 5)
        self.assertIsNotNone(id_map)

        for i in range(5):
            _otf2.IdMap_AddIdPair(id_map, ids[i], ids[i])

        for i in range(5):
            global_id = _otf2.IdMap_GetGlobalId(id_map, i)
            self.assertEqual(i, global_id)

        _otf2.IdMap_Free(id_map)


if __name__ == '__main__':
    unittest.main()




# Generated by OTF2 Template Engine

import _otf2

from .enums import Type


def parse(registry, union_value, type_id):
    if type_id == Type.UINT8:
        return union_value.uint8
    if type_id == Type.UINT16:
        return union_value.uint16
    if type_id == Type.UINT32:
        return union_value.uint32
    if type_id == Type.UINT64:
        return union_value.uint64
    if type_id == Type.INT8:
        return union_value.int8
    if type_id == Type.INT16:
        return union_value.int16
    if type_id == Type.INT32:
        return union_value.int32
    if type_id == Type.INT64:
        return union_value.int64
    if type_id == Type.FLOAT:
        return union_value.float32
    if type_id == Type.DOUBLE:
        return union_value.float64
    if type_id == Type.STRING:
        return registry.strings[union_value.stringRef]
    if type_id == Type.ATTRIBUTE:
        return registry.attributes[union_value.attributeRef]
    if type_id == Type.LOCATION:
        return registry.locations[union_value.locationRef]
    if type_id == Type.REGION:
        return registry.regions[union_value.regionRef]
    if type_id == Type.GROUP:
        return registry.groups[union_value.groupRef]
    if type_id == Type.METRIC:
        return registry.metrics[union_value.metricRef]
    if type_id == Type.COMM:
        return registry.comms[union_value.commRef]
    if type_id == Type.PARAMETER:
        return registry.parameters[union_value.parameterRef]
    if type_id == Type.RMA_WIN:
        return registry.rma_wins[union_value.rmaWinRef]
    if type_id == Type.SOURCE_CODE_LOCATION:
        return registry.source_code_locations[union_value.sourceCodeLocationRef]
    if type_id == Type.CALLING_CONTEXT:
        return registry.calling_contexts[union_value.callingContextRef]
    if type_id == Type.INTERRUPT_GENERATOR:
        return registry.interrupt_generators[union_value.interruptGeneratorRef]
    if type_id == Type.IO_FILE:
        return registry.io_files[union_value.ioFileRef]
    if type_id == Type.IO_HANDLE:
        return registry.io_handles[union_value.ioHandleRef]
    if type_id == Type.LOCATION_GROUP:
        return registry.location_groups[union_value.locationGroupRef]
    raise TypeError("Unexpected type.")


def make(registry, value, type_id, writer=None):
    """
    :param registry:
    :param value:
    :param otf2.Type type_id:
    :param writer: If set, the string value will be written. Otherwise just added to the registry
    :return:
    """
    # values are expected to be int, float, or a Definition subclass
    union = _otf2.AttributeValue()
    if type_id == Type.UINT8:
        union.uint8 = int(value)
    elif type_id == Type.UINT16:
        union.uint16 = int(value)
    elif type_id == Type.UINT32:
        union.uint32 = int(value)
    elif type_id == Type.UINT64:
        union.uint64 = int(value)
    elif type_id == Type.INT8:
        union.int8 = int(value)
    elif type_id == Type.INT16:
        union.int16 = int(value)
    elif type_id == Type.INT32:
        union.int32 = int(value)
    elif type_id == Type.INT64:
        union.int64 = int(value)
    elif type_id == Type.FLOAT:
        union.float32 = float(value)
    elif type_id == Type.DOUBLE:
        union.float64 = float(value)
    elif type_id == Type.STRING:
        union.stringRef = registry.strings.get_ref(value, writer)
    elif type_id == Type.ATTRIBUTE:
        union.attributeRef = value._ref
    elif type_id == Type.LOCATION:
        union.locationRef = value._ref
    elif type_id == Type.REGION:
        union.regionRef = value._ref
    elif type_id == Type.GROUP:
        union.groupRef = value._ref
    elif type_id == Type.METRIC:
        union.metricRef = value._ref
    elif type_id == Type.COMM:
        union.commRef = value._ref
    elif type_id == Type.PARAMETER:
        union.parameterRef = value._ref
    elif type_id == Type.RMA_WIN:
        union.rmaWinRef = value._ref
    elif type_id == Type.SOURCE_CODE_LOCATION:
        union.sourceCodeLocationRef = value._ref
    elif type_id == Type.CALLING_CONTEXT:
        union.callingContextRef = value._ref
    elif type_id == Type.INTERRUPT_GENERATOR:
        union.interruptGeneratorRef = value._ref
    elif type_id == Type.IO_FILE:
        union.ioFileRef = value._ref
    elif type_id == Type.IO_HANDLE:
        union.ioHandleRef = value._ref
    elif type_id == Type.LOCATION_GROUP:
        union.locationGroupRef = value._ref
    else:
        raise TypeError("Unexpected type id given")

    return union


class AttributeValue(object):
    def __init__(self, value, type_id):
        self.value = value
        self.type_id = type_id

    def as_otf2(self, registry, writer=None):
        return make(registry, self.value, self.type_id, writer)

    def set(self, value, type_id=None):
        # TODO Ideally we would want to check if the value is compatible with the type,
        # but that would introduce a circular dependency to the definitions
        self.value = value
        if type_id is not None:
            self.type_id = type_id

    @classmethod
    def _construct(cls, registry, union_value, type_id):
        return cls(parse(registry, union_value, type_id), type_id)




""" Generated by OTF2 Template Engine """

import ctypes

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .Definitions import *
from .Events import *


class AttributeValue(ctypes.Union):
    """
    ctypes does not support unions as (non-pointer) arguments
    _arg_value / _arg_type is a crude and evil hack to guess
    the right argument type. This type must be correct in terms of
    size (maximum of the union) and
    calling convention (architecture specific, but for AMD64 int usually wins).
    Whenever used as a direct function argument (also applies to callbacks)
    this type should be used instead of the union. Otherwise you get undefined
    behavior.
    """
    _arg_type = ctypes.c_uint64
    _fields_ = [
        ("uint8", ctypes.c_uint8),
        ("uint16", ctypes.c_uint16),
        ("uint32", ctypes.c_uint32),
        ("uint64", ctypes.c_uint64),
        ("int8", ctypes.c_int8),
        ("int16", ctypes.c_int16),
        ("int32", ctypes.c_int32),
        ("int64", ctypes.c_int64),
        ("float32", ctypes.c_float),
        ("float64", ctypes.c_double),
        ("stringRef", StringRef),
        ("attributeRef", AttributeRef),
        ("locationRef", LocationRef),
        ("regionRef", RegionRef),
        ("groupRef", GroupRef),
        ("metricRef", MetricRef),
        ("commRef", CommRef),
        ("parameterRef", ParameterRef),
        ("rmaWinRef", RmaWinRef),
        ("sourceCodeLocationRef", SourceCodeLocationRef),
        ("callingContextRef", CallingContextRef),
        ("interruptGeneratorRef", InterruptGeneratorRef),
        ("ioFileRef", IoFileRef),
        ("ioHandleRef", IoHandleRef),
        ("locationGroupRef", LocationGroupRef),
        ("_arg_value", ctypes.c_uint64),
    ]

def AttributeValue_SetBoolean(enumValue, type_=Type(), value=AttributeValue()):
    c_SetBoolean = conf.lib.OTF2_AttributeValue_SetBoolean
    c_SetBoolean.argtypes = [ Boolean, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetBoolean.restype = ErrorCode
    c_SetBoolean.errcheck = HandleErrorCode
    c_SetBoolean(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetFileType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetFileType = conf.lib.OTF2_AttributeValue_SetFileType
    c_SetFileType.argtypes = [ FileType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetFileType.restype = ErrorCode
    c_SetFileType.errcheck = HandleErrorCode
    c_SetFileType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetFileSubstrate(enumValue, type_=Type(), value=AttributeValue()):
    c_SetFileSubstrate = conf.lib.OTF2_AttributeValue_SetFileSubstrate
    c_SetFileSubstrate.argtypes = [ FileSubstrate, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetFileSubstrate.restype = ErrorCode
    c_SetFileSubstrate.errcheck = HandleErrorCode
    c_SetFileSubstrate(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMappingType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMappingType = conf.lib.OTF2_AttributeValue_SetMappingType
    c_SetMappingType.argtypes = [ MappingType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMappingType.restype = ErrorCode
    c_SetMappingType.errcheck = HandleErrorCode
    c_SetMappingType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetType = conf.lib.OTF2_AttributeValue_SetType
    c_SetType.argtypes = [ Type, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetType.restype = ErrorCode
    c_SetType.errcheck = HandleErrorCode
    c_SetType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetParadigm(enumValue, type_=Type(), value=AttributeValue()):
    c_SetParadigm = conf.lib.OTF2_AttributeValue_SetParadigm
    c_SetParadigm.argtypes = [ Paradigm, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetParadigm.restype = ErrorCode
    c_SetParadigm.errcheck = HandleErrorCode
    c_SetParadigm(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetParadigmClass(enumValue, type_=Type(), value=AttributeValue()):
    c_SetParadigmClass = conf.lib.OTF2_AttributeValue_SetParadigmClass
    c_SetParadigmClass.argtypes = [ ParadigmClass, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetParadigmClass.restype = ErrorCode
    c_SetParadigmClass.errcheck = HandleErrorCode
    c_SetParadigmClass(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetParadigmProperty(enumValue, type_=Type(), value=AttributeValue()):
    c_SetParadigmProperty = conf.lib.OTF2_AttributeValue_SetParadigmProperty
    c_SetParadigmProperty.argtypes = [ ParadigmProperty, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetParadigmProperty.restype = ErrorCode
    c_SetParadigmProperty.errcheck = HandleErrorCode
    c_SetParadigmProperty(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetThumbnailType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetThumbnailType = conf.lib.OTF2_AttributeValue_SetThumbnailType
    c_SetThumbnailType.argtypes = [ ThumbnailType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetThumbnailType.restype = ErrorCode
    c_SetThumbnailType.errcheck = HandleErrorCode
    c_SetThumbnailType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetSystemTreeDomain(enumValue, type_=Type(), value=AttributeValue()):
    c_SetSystemTreeDomain = conf.lib.OTF2_AttributeValue_SetSystemTreeDomain
    c_SetSystemTreeDomain.argtypes = [ SystemTreeDomain, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetSystemTreeDomain.restype = ErrorCode
    c_SetSystemTreeDomain.errcheck = HandleErrorCode
    c_SetSystemTreeDomain(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetLocationGroupType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetLocationGroupType = conf.lib.OTF2_AttributeValue_SetLocationGroupType
    c_SetLocationGroupType.argtypes = [ LocationGroupType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetLocationGroupType.restype = ErrorCode
    c_SetLocationGroupType.errcheck = HandleErrorCode
    c_SetLocationGroupType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetLocationType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetLocationType = conf.lib.OTF2_AttributeValue_SetLocationType
    c_SetLocationType.argtypes = [ LocationType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetLocationType.restype = ErrorCode
    c_SetLocationType.errcheck = HandleErrorCode
    c_SetLocationType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRegionRole(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRegionRole = conf.lib.OTF2_AttributeValue_SetRegionRole
    c_SetRegionRole.argtypes = [ RegionRole, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRegionRole.restype = ErrorCode
    c_SetRegionRole.errcheck = HandleErrorCode
    c_SetRegionRole(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRegionFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRegionFlag = conf.lib.OTF2_AttributeValue_SetRegionFlag
    c_SetRegionFlag.argtypes = [ RegionFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRegionFlag.restype = ErrorCode
    c_SetRegionFlag.errcheck = HandleErrorCode
    c_SetRegionFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetGroupType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetGroupType = conf.lib.OTF2_AttributeValue_SetGroupType
    c_SetGroupType.argtypes = [ GroupType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetGroupType.restype = ErrorCode
    c_SetGroupType.errcheck = HandleErrorCode
    c_SetGroupType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetGroupFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetGroupFlag = conf.lib.OTF2_AttributeValue_SetGroupFlag
    c_SetGroupFlag.argtypes = [ GroupFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetGroupFlag.restype = ErrorCode
    c_SetGroupFlag.errcheck = HandleErrorCode
    c_SetGroupFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetBase(enumValue, type_=Type(), value=AttributeValue()):
    c_SetBase = conf.lib.OTF2_AttributeValue_SetBase
    c_SetBase.argtypes = [ Base, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetBase.restype = ErrorCode
    c_SetBase.errcheck = HandleErrorCode
    c_SetBase(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricOccurrence(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricOccurrence = conf.lib.OTF2_AttributeValue_SetMetricOccurrence
    c_SetMetricOccurrence.argtypes = [ MetricOccurrence, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricOccurrence.restype = ErrorCode
    c_SetMetricOccurrence.errcheck = HandleErrorCode
    c_SetMetricOccurrence(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricType = conf.lib.OTF2_AttributeValue_SetMetricType
    c_SetMetricType.argtypes = [ MetricType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricType.restype = ErrorCode
    c_SetMetricType.errcheck = HandleErrorCode
    c_SetMetricType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricValueProperty(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricValueProperty = conf.lib.OTF2_AttributeValue_SetMetricValueProperty
    c_SetMetricValueProperty.argtypes = [ MetricValueProperty, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricValueProperty.restype = ErrorCode
    c_SetMetricValueProperty.errcheck = HandleErrorCode
    c_SetMetricValueProperty(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricTiming(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricTiming = conf.lib.OTF2_AttributeValue_SetMetricTiming
    c_SetMetricTiming.argtypes = [ MetricTiming, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricTiming.restype = ErrorCode
    c_SetMetricTiming.errcheck = HandleErrorCode
    c_SetMetricTiming(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricMode(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricMode = conf.lib.OTF2_AttributeValue_SetMetricMode
    c_SetMetricMode.argtypes = [ MetricMode, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricMode.restype = ErrorCode
    c_SetMetricMode.errcheck = HandleErrorCode
    c_SetMetricMode(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMetricScope(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMetricScope = conf.lib.OTF2_AttributeValue_SetMetricScope
    c_SetMetricScope.argtypes = [ MetricScope, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMetricScope.restype = ErrorCode
    c_SetMetricScope.errcheck = HandleErrorCode
    c_SetMetricScope(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRecorderKind(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRecorderKind = conf.lib.OTF2_AttributeValue_SetRecorderKind
    c_SetRecorderKind.argtypes = [ RecorderKind, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRecorderKind.restype = ErrorCode
    c_SetRecorderKind.errcheck = HandleErrorCode
    c_SetRecorderKind(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetParameterType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetParameterType = conf.lib.OTF2_AttributeValue_SetParameterType
    c_SetParameterType.argtypes = [ ParameterType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetParameterType.restype = ErrorCode
    c_SetParameterType.errcheck = HandleErrorCode
    c_SetParameterType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetCartPeriodicity(enumValue, type_=Type(), value=AttributeValue()):
    c_SetCartPeriodicity = conf.lib.OTF2_AttributeValue_SetCartPeriodicity
    c_SetCartPeriodicity.argtypes = [ CartPeriodicity, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetCartPeriodicity.restype = ErrorCode
    c_SetCartPeriodicity.errcheck = HandleErrorCode
    c_SetCartPeriodicity(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetInterruptGeneratorMode(enumValue, type_=Type(), value=AttributeValue()):
    c_SetInterruptGeneratorMode = conf.lib.OTF2_AttributeValue_SetInterruptGeneratorMode
    c_SetInterruptGeneratorMode.argtypes = [ InterruptGeneratorMode, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetInterruptGeneratorMode.restype = ErrorCode
    c_SetInterruptGeneratorMode.errcheck = HandleErrorCode
    c_SetInterruptGeneratorMode(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetMeasurementMode(enumValue, type_=Type(), value=AttributeValue()):
    c_SetMeasurementMode = conf.lib.OTF2_AttributeValue_SetMeasurementMode
    c_SetMeasurementMode.argtypes = [ MeasurementMode, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetMeasurementMode.restype = ErrorCode
    c_SetMeasurementMode.errcheck = HandleErrorCode
    c_SetMeasurementMode(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetCollectiveOp(enumValue, type_=Type(), value=AttributeValue()):
    c_SetCollectiveOp = conf.lib.OTF2_AttributeValue_SetCollectiveOp
    c_SetCollectiveOp.argtypes = [ CollectiveOp, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetCollectiveOp.restype = ErrorCode
    c_SetCollectiveOp.errcheck = HandleErrorCode
    c_SetCollectiveOp(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRmaSyncType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRmaSyncType = conf.lib.OTF2_AttributeValue_SetRmaSyncType
    c_SetRmaSyncType.argtypes = [ RmaSyncType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRmaSyncType.restype = ErrorCode
    c_SetRmaSyncType.errcheck = HandleErrorCode
    c_SetRmaSyncType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRmaSyncLevel(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRmaSyncLevel = conf.lib.OTF2_AttributeValue_SetRmaSyncLevel
    c_SetRmaSyncLevel.argtypes = [ RmaSyncLevel, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRmaSyncLevel.restype = ErrorCode
    c_SetRmaSyncLevel.errcheck = HandleErrorCode
    c_SetRmaSyncLevel(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetLockType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetLockType = conf.lib.OTF2_AttributeValue_SetLockType
    c_SetLockType.argtypes = [ LockType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetLockType.restype = ErrorCode
    c_SetLockType.errcheck = HandleErrorCode
    c_SetLockType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRmaAtomicType(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRmaAtomicType = conf.lib.OTF2_AttributeValue_SetRmaAtomicType
    c_SetRmaAtomicType.argtypes = [ RmaAtomicType, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRmaAtomicType.restype = ErrorCode
    c_SetRmaAtomicType.errcheck = HandleErrorCode
    c_SetRmaAtomicType(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoParadigmClass(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoParadigmClass = conf.lib.OTF2_AttributeValue_SetIoParadigmClass
    c_SetIoParadigmClass.argtypes = [ IoParadigmClass, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoParadigmClass.restype = ErrorCode
    c_SetIoParadigmClass.errcheck = HandleErrorCode
    c_SetIoParadigmClass(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoParadigmFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoParadigmFlag = conf.lib.OTF2_AttributeValue_SetIoParadigmFlag
    c_SetIoParadigmFlag.argtypes = [ IoParadigmFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoParadigmFlag.restype = ErrorCode
    c_SetIoParadigmFlag.errcheck = HandleErrorCode
    c_SetIoParadigmFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoParadigmProperty(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoParadigmProperty = conf.lib.OTF2_AttributeValue_SetIoParadigmProperty
    c_SetIoParadigmProperty.argtypes = [ IoParadigmProperty, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoParadigmProperty.restype = ErrorCode
    c_SetIoParadigmProperty.errcheck = HandleErrorCode
    c_SetIoParadigmProperty(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoHandleFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoHandleFlag = conf.lib.OTF2_AttributeValue_SetIoHandleFlag
    c_SetIoHandleFlag.argtypes = [ IoHandleFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoHandleFlag.restype = ErrorCode
    c_SetIoHandleFlag.errcheck = HandleErrorCode
    c_SetIoHandleFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoAccessMode(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoAccessMode = conf.lib.OTF2_AttributeValue_SetIoAccessMode
    c_SetIoAccessMode.argtypes = [ IoAccessMode, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoAccessMode.restype = ErrorCode
    c_SetIoAccessMode.errcheck = HandleErrorCode
    c_SetIoAccessMode(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoCreationFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoCreationFlag = conf.lib.OTF2_AttributeValue_SetIoCreationFlag
    c_SetIoCreationFlag.argtypes = [ IoCreationFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoCreationFlag.restype = ErrorCode
    c_SetIoCreationFlag.errcheck = HandleErrorCode
    c_SetIoCreationFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoStatusFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoStatusFlag = conf.lib.OTF2_AttributeValue_SetIoStatusFlag
    c_SetIoStatusFlag.argtypes = [ IoStatusFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoStatusFlag.restype = ErrorCode
    c_SetIoStatusFlag.errcheck = HandleErrorCode
    c_SetIoStatusFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoSeekOption(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoSeekOption = conf.lib.OTF2_AttributeValue_SetIoSeekOption
    c_SetIoSeekOption.argtypes = [ IoSeekOption, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoSeekOption.restype = ErrorCode
    c_SetIoSeekOption.errcheck = HandleErrorCode
    c_SetIoSeekOption(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoOperationMode(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoOperationMode = conf.lib.OTF2_AttributeValue_SetIoOperationMode
    c_SetIoOperationMode.argtypes = [ IoOperationMode, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoOperationMode.restype = ErrorCode
    c_SetIoOperationMode.errcheck = HandleErrorCode
    c_SetIoOperationMode(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetIoOperationFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetIoOperationFlag = conf.lib.OTF2_AttributeValue_SetIoOperationFlag
    c_SetIoOperationFlag.argtypes = [ IoOperationFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetIoOperationFlag.restype = ErrorCode
    c_SetIoOperationFlag.errcheck = HandleErrorCode
    c_SetIoOperationFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetCommFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetCommFlag = conf.lib.OTF2_AttributeValue_SetCommFlag
    c_SetCommFlag.argtypes = [ CommFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetCommFlag.restype = ErrorCode
    c_SetCommFlag.errcheck = HandleErrorCode
    c_SetCommFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetRmaWinFlag(enumValue, type_=Type(), value=AttributeValue()):
    c_SetRmaWinFlag = conf.lib.OTF2_AttributeValue_SetRmaWinFlag
    c_SetRmaWinFlag.argtypes = [ RmaWinFlag, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetRmaWinFlag.restype = ErrorCode
    c_SetRmaWinFlag.errcheck = HandleErrorCode
    c_SetRmaWinFlag(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_SetCollectiveRoot(enumValue, type_=Type(), value=AttributeValue()):
    c_SetCollectiveRoot = conf.lib.OTF2_AttributeValue_SetCollectiveRoot
    c_SetCollectiveRoot.argtypes = [ CollectiveRoot, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_SetCollectiveRoot.restype = ErrorCode
    c_SetCollectiveRoot.errcheck = HandleErrorCode
    c_SetCollectiveRoot(enumValue, ctypes.byref(type_), ctypes.byref(value))
    return (type_, value)

def AttributeValue_GetBoolean(type, enum_value=Boolean()):
    c_GetBoolean = conf.lib.OTF2_AttributeValue_GetBoolean
    c_GetBoolean.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(Boolean) ]
    c_GetBoolean.restype = ErrorCode
    c_GetBoolean.errcheck = HandleErrorCode
    c_GetBoolean(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetFileType(type, enum_value=FileType()):
    c_GetFileType = conf.lib.OTF2_AttributeValue_GetFileType
    c_GetFileType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(FileType) ]
    c_GetFileType.restype = ErrorCode
    c_GetFileType.errcheck = HandleErrorCode
    c_GetFileType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetFileSubstrate(type, enum_value=FileSubstrate()):
    c_GetFileSubstrate = conf.lib.OTF2_AttributeValue_GetFileSubstrate
    c_GetFileSubstrate.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(FileSubstrate) ]
    c_GetFileSubstrate.restype = ErrorCode
    c_GetFileSubstrate.errcheck = HandleErrorCode
    c_GetFileSubstrate(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMappingType(type, enum_value=MappingType()):
    c_GetMappingType = conf.lib.OTF2_AttributeValue_GetMappingType
    c_GetMappingType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MappingType) ]
    c_GetMappingType.restype = ErrorCode
    c_GetMappingType.errcheck = HandleErrorCode
    c_GetMappingType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetType(type, enum_value=Type()):
    c_GetType = conf.lib.OTF2_AttributeValue_GetType
    c_GetType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(Type) ]
    c_GetType.restype = ErrorCode
    c_GetType.errcheck = HandleErrorCode
    c_GetType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetParadigm(type, enum_value=Paradigm()):
    c_GetParadigm = conf.lib.OTF2_AttributeValue_GetParadigm
    c_GetParadigm.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(Paradigm) ]
    c_GetParadigm.restype = ErrorCode
    c_GetParadigm.errcheck = HandleErrorCode
    c_GetParadigm(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetParadigmClass(type, enum_value=ParadigmClass()):
    c_GetParadigmClass = conf.lib.OTF2_AttributeValue_GetParadigmClass
    c_GetParadigmClass.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(ParadigmClass) ]
    c_GetParadigmClass.restype = ErrorCode
    c_GetParadigmClass.errcheck = HandleErrorCode
    c_GetParadigmClass(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetParadigmProperty(type, enum_value=ParadigmProperty()):
    c_GetParadigmProperty = conf.lib.OTF2_AttributeValue_GetParadigmProperty
    c_GetParadigmProperty.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(ParadigmProperty) ]
    c_GetParadigmProperty.restype = ErrorCode
    c_GetParadigmProperty.errcheck = HandleErrorCode
    c_GetParadigmProperty(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetThumbnailType(type, enum_value=ThumbnailType()):
    c_GetThumbnailType = conf.lib.OTF2_AttributeValue_GetThumbnailType
    c_GetThumbnailType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(ThumbnailType) ]
    c_GetThumbnailType.restype = ErrorCode
    c_GetThumbnailType.errcheck = HandleErrorCode
    c_GetThumbnailType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetSystemTreeDomain(type, enum_value=SystemTreeDomain()):
    c_GetSystemTreeDomain = conf.lib.OTF2_AttributeValue_GetSystemTreeDomain
    c_GetSystemTreeDomain.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(SystemTreeDomain) ]
    c_GetSystemTreeDomain.restype = ErrorCode
    c_GetSystemTreeDomain.errcheck = HandleErrorCode
    c_GetSystemTreeDomain(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetLocationGroupType(type, enum_value=LocationGroupType()):
    c_GetLocationGroupType = conf.lib.OTF2_AttributeValue_GetLocationGroupType
    c_GetLocationGroupType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(LocationGroupType) ]
    c_GetLocationGroupType.restype = ErrorCode
    c_GetLocationGroupType.errcheck = HandleErrorCode
    c_GetLocationGroupType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetLocationType(type, enum_value=LocationType()):
    c_GetLocationType = conf.lib.OTF2_AttributeValue_GetLocationType
    c_GetLocationType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(LocationType) ]
    c_GetLocationType.restype = ErrorCode
    c_GetLocationType.errcheck = HandleErrorCode
    c_GetLocationType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRegionRole(type, enum_value=RegionRole()):
    c_GetRegionRole = conf.lib.OTF2_AttributeValue_GetRegionRole
    c_GetRegionRole.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RegionRole) ]
    c_GetRegionRole.restype = ErrorCode
    c_GetRegionRole.errcheck = HandleErrorCode
    c_GetRegionRole(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRegionFlag(type, enum_value=RegionFlag()):
    c_GetRegionFlag = conf.lib.OTF2_AttributeValue_GetRegionFlag
    c_GetRegionFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RegionFlag) ]
    c_GetRegionFlag.restype = ErrorCode
    c_GetRegionFlag.errcheck = HandleErrorCode
    c_GetRegionFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetGroupType(type, enum_value=GroupType()):
    c_GetGroupType = conf.lib.OTF2_AttributeValue_GetGroupType
    c_GetGroupType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(GroupType) ]
    c_GetGroupType.restype = ErrorCode
    c_GetGroupType.errcheck = HandleErrorCode
    c_GetGroupType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetGroupFlag(type, enum_value=GroupFlag()):
    c_GetGroupFlag = conf.lib.OTF2_AttributeValue_GetGroupFlag
    c_GetGroupFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(GroupFlag) ]
    c_GetGroupFlag.restype = ErrorCode
    c_GetGroupFlag.errcheck = HandleErrorCode
    c_GetGroupFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetBase(type, enum_value=Base()):
    c_GetBase = conf.lib.OTF2_AttributeValue_GetBase
    c_GetBase.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(Base) ]
    c_GetBase.restype = ErrorCode
    c_GetBase.errcheck = HandleErrorCode
    c_GetBase(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricOccurrence(type, enum_value=MetricOccurrence()):
    c_GetMetricOccurrence = conf.lib.OTF2_AttributeValue_GetMetricOccurrence
    c_GetMetricOccurrence.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricOccurrence) ]
    c_GetMetricOccurrence.restype = ErrorCode
    c_GetMetricOccurrence.errcheck = HandleErrorCode
    c_GetMetricOccurrence(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricType(type, enum_value=MetricType()):
    c_GetMetricType = conf.lib.OTF2_AttributeValue_GetMetricType
    c_GetMetricType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricType) ]
    c_GetMetricType.restype = ErrorCode
    c_GetMetricType.errcheck = HandleErrorCode
    c_GetMetricType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricValueProperty(type, enum_value=MetricValueProperty()):
    c_GetMetricValueProperty = conf.lib.OTF2_AttributeValue_GetMetricValueProperty
    c_GetMetricValueProperty.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricValueProperty) ]
    c_GetMetricValueProperty.restype = ErrorCode
    c_GetMetricValueProperty.errcheck = HandleErrorCode
    c_GetMetricValueProperty(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricTiming(type, enum_value=MetricTiming()):
    c_GetMetricTiming = conf.lib.OTF2_AttributeValue_GetMetricTiming
    c_GetMetricTiming.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricTiming) ]
    c_GetMetricTiming.restype = ErrorCode
    c_GetMetricTiming.errcheck = HandleErrorCode
    c_GetMetricTiming(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricMode(type, enum_value=MetricMode()):
    c_GetMetricMode = conf.lib.OTF2_AttributeValue_GetMetricMode
    c_GetMetricMode.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricMode) ]
    c_GetMetricMode.restype = ErrorCode
    c_GetMetricMode.errcheck = HandleErrorCode
    c_GetMetricMode(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMetricScope(type, enum_value=MetricScope()):
    c_GetMetricScope = conf.lib.OTF2_AttributeValue_GetMetricScope
    c_GetMetricScope.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MetricScope) ]
    c_GetMetricScope.restype = ErrorCode
    c_GetMetricScope.errcheck = HandleErrorCode
    c_GetMetricScope(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRecorderKind(type, enum_value=RecorderKind()):
    c_GetRecorderKind = conf.lib.OTF2_AttributeValue_GetRecorderKind
    c_GetRecorderKind.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RecorderKind) ]
    c_GetRecorderKind.restype = ErrorCode
    c_GetRecorderKind.errcheck = HandleErrorCode
    c_GetRecorderKind(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetParameterType(type, enum_value=ParameterType()):
    c_GetParameterType = conf.lib.OTF2_AttributeValue_GetParameterType
    c_GetParameterType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(ParameterType) ]
    c_GetParameterType.restype = ErrorCode
    c_GetParameterType.errcheck = HandleErrorCode
    c_GetParameterType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetCartPeriodicity(type, enum_value=CartPeriodicity()):
    c_GetCartPeriodicity = conf.lib.OTF2_AttributeValue_GetCartPeriodicity
    c_GetCartPeriodicity.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(CartPeriodicity) ]
    c_GetCartPeriodicity.restype = ErrorCode
    c_GetCartPeriodicity.errcheck = HandleErrorCode
    c_GetCartPeriodicity(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetInterruptGeneratorMode(type, enum_value=InterruptGeneratorMode()):
    c_GetInterruptGeneratorMode = conf.lib.OTF2_AttributeValue_GetInterruptGeneratorMode
    c_GetInterruptGeneratorMode.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(InterruptGeneratorMode) ]
    c_GetInterruptGeneratorMode.restype = ErrorCode
    c_GetInterruptGeneratorMode.errcheck = HandleErrorCode
    c_GetInterruptGeneratorMode(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetMeasurementMode(type, enum_value=MeasurementMode()):
    c_GetMeasurementMode = conf.lib.OTF2_AttributeValue_GetMeasurementMode
    c_GetMeasurementMode.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(MeasurementMode) ]
    c_GetMeasurementMode.restype = ErrorCode
    c_GetMeasurementMode.errcheck = HandleErrorCode
    c_GetMeasurementMode(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetCollectiveOp(type, enum_value=CollectiveOp()):
    c_GetCollectiveOp = conf.lib.OTF2_AttributeValue_GetCollectiveOp
    c_GetCollectiveOp.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(CollectiveOp) ]
    c_GetCollectiveOp.restype = ErrorCode
    c_GetCollectiveOp.errcheck = HandleErrorCode
    c_GetCollectiveOp(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRmaSyncType(type, enum_value=RmaSyncType()):
    c_GetRmaSyncType = conf.lib.OTF2_AttributeValue_GetRmaSyncType
    c_GetRmaSyncType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RmaSyncType) ]
    c_GetRmaSyncType.restype = ErrorCode
    c_GetRmaSyncType.errcheck = HandleErrorCode
    c_GetRmaSyncType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRmaSyncLevel(type, enum_value=RmaSyncLevel()):
    c_GetRmaSyncLevel = conf.lib.OTF2_AttributeValue_GetRmaSyncLevel
    c_GetRmaSyncLevel.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RmaSyncLevel) ]
    c_GetRmaSyncLevel.restype = ErrorCode
    c_GetRmaSyncLevel.errcheck = HandleErrorCode
    c_GetRmaSyncLevel(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetLockType(type, enum_value=LockType()):
    c_GetLockType = conf.lib.OTF2_AttributeValue_GetLockType
    c_GetLockType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(LockType) ]
    c_GetLockType.restype = ErrorCode
    c_GetLockType.errcheck = HandleErrorCode
    c_GetLockType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRmaAtomicType(type, enum_value=RmaAtomicType()):
    c_GetRmaAtomicType = conf.lib.OTF2_AttributeValue_GetRmaAtomicType
    c_GetRmaAtomicType.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RmaAtomicType) ]
    c_GetRmaAtomicType.restype = ErrorCode
    c_GetRmaAtomicType.errcheck = HandleErrorCode
    c_GetRmaAtomicType(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoParadigmClass(type, enum_value=IoParadigmClass()):
    c_GetIoParadigmClass = conf.lib.OTF2_AttributeValue_GetIoParadigmClass
    c_GetIoParadigmClass.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoParadigmClass) ]
    c_GetIoParadigmClass.restype = ErrorCode
    c_GetIoParadigmClass.errcheck = HandleErrorCode
    c_GetIoParadigmClass(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoParadigmFlag(type, enum_value=IoParadigmFlag()):
    c_GetIoParadigmFlag = conf.lib.OTF2_AttributeValue_GetIoParadigmFlag
    c_GetIoParadigmFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoParadigmFlag) ]
    c_GetIoParadigmFlag.restype = ErrorCode
    c_GetIoParadigmFlag.errcheck = HandleErrorCode
    c_GetIoParadigmFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoParadigmProperty(type, enum_value=IoParadigmProperty()):
    c_GetIoParadigmProperty = conf.lib.OTF2_AttributeValue_GetIoParadigmProperty
    c_GetIoParadigmProperty.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoParadigmProperty) ]
    c_GetIoParadigmProperty.restype = ErrorCode
    c_GetIoParadigmProperty.errcheck = HandleErrorCode
    c_GetIoParadigmProperty(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoHandleFlag(type, enum_value=IoHandleFlag()):
    c_GetIoHandleFlag = conf.lib.OTF2_AttributeValue_GetIoHandleFlag
    c_GetIoHandleFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoHandleFlag) ]
    c_GetIoHandleFlag.restype = ErrorCode
    c_GetIoHandleFlag.errcheck = HandleErrorCode
    c_GetIoHandleFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoAccessMode(type, enum_value=IoAccessMode()):
    c_GetIoAccessMode = conf.lib.OTF2_AttributeValue_GetIoAccessMode
    c_GetIoAccessMode.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoAccessMode) ]
    c_GetIoAccessMode.restype = ErrorCode
    c_GetIoAccessMode.errcheck = HandleErrorCode
    c_GetIoAccessMode(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoCreationFlag(type, enum_value=IoCreationFlag()):
    c_GetIoCreationFlag = conf.lib.OTF2_AttributeValue_GetIoCreationFlag
    c_GetIoCreationFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoCreationFlag) ]
    c_GetIoCreationFlag.restype = ErrorCode
    c_GetIoCreationFlag.errcheck = HandleErrorCode
    c_GetIoCreationFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoStatusFlag(type, enum_value=IoStatusFlag()):
    c_GetIoStatusFlag = conf.lib.OTF2_AttributeValue_GetIoStatusFlag
    c_GetIoStatusFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoStatusFlag) ]
    c_GetIoStatusFlag.restype = ErrorCode
    c_GetIoStatusFlag.errcheck = HandleErrorCode
    c_GetIoStatusFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoSeekOption(type, enum_value=IoSeekOption()):
    c_GetIoSeekOption = conf.lib.OTF2_AttributeValue_GetIoSeekOption
    c_GetIoSeekOption.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoSeekOption) ]
    c_GetIoSeekOption.restype = ErrorCode
    c_GetIoSeekOption.errcheck = HandleErrorCode
    c_GetIoSeekOption(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoOperationMode(type, enum_value=IoOperationMode()):
    c_GetIoOperationMode = conf.lib.OTF2_AttributeValue_GetIoOperationMode
    c_GetIoOperationMode.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoOperationMode) ]
    c_GetIoOperationMode.restype = ErrorCode
    c_GetIoOperationMode.errcheck = HandleErrorCode
    c_GetIoOperationMode(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetIoOperationFlag(type, enum_value=IoOperationFlag()):
    c_GetIoOperationFlag = conf.lib.OTF2_AttributeValue_GetIoOperationFlag
    c_GetIoOperationFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(IoOperationFlag) ]
    c_GetIoOperationFlag.restype = ErrorCode
    c_GetIoOperationFlag.errcheck = HandleErrorCode
    c_GetIoOperationFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetCommFlag(type, enum_value=CommFlag()):
    c_GetCommFlag = conf.lib.OTF2_AttributeValue_GetCommFlag
    c_GetCommFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(CommFlag) ]
    c_GetCommFlag.restype = ErrorCode
    c_GetCommFlag.errcheck = HandleErrorCode
    c_GetCommFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetRmaWinFlag(type, enum_value=RmaWinFlag()):
    c_GetRmaWinFlag = conf.lib.OTF2_AttributeValue_GetRmaWinFlag
    c_GetRmaWinFlag.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(RmaWinFlag) ]
    c_GetRmaWinFlag.restype = ErrorCode
    c_GetRmaWinFlag.errcheck = HandleErrorCode
    c_GetRmaWinFlag(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

def AttributeValue_GetCollectiveRoot(type, enum_value=CollectiveRoot()):
    c_GetCollectiveRoot = conf.lib.OTF2_AttributeValue_GetCollectiveRoot
    c_GetCollectiveRoot.argtypes = [ Type, AttributeValue._arg_type, ctypes.POINTER(CollectiveRoot) ]
    c_GetCollectiveRoot.restype = ErrorCode
    c_GetCollectiveRoot.errcheck = HandleErrorCode
    c_GetCollectiveRoot(type, value._arg_value, ctypes.byref(enum_value))
    return enum_value

__all__ = [
    'AttributeValue',
    'AttributeValue_SetBoolean',
    'AttributeValue_GetBoolean',
    'AttributeValue_SetFileType',
    'AttributeValue_GetFileType',
    'AttributeValue_SetFileSubstrate',
    'AttributeValue_GetFileSubstrate',
    'AttributeValue_SetMappingType',
    'AttributeValue_GetMappingType',
    'AttributeValue_SetType',
    'AttributeValue_GetType',
    'AttributeValue_SetParadigm',
    'AttributeValue_GetParadigm',
    'AttributeValue_SetParadigmClass',
    'AttributeValue_GetParadigmClass',
    'AttributeValue_SetParadigmProperty',
    'AttributeValue_GetParadigmProperty',
    'AttributeValue_SetThumbnailType',
    'AttributeValue_GetThumbnailType',
    'AttributeValue_SetSystemTreeDomain',
    'AttributeValue_GetSystemTreeDomain',
    'AttributeValue_SetLocationGroupType',
    'AttributeValue_GetLocationGroupType',
    'AttributeValue_SetLocationType',
    'AttributeValue_GetLocationType',
    'AttributeValue_SetRegionRole',
    'AttributeValue_GetRegionRole',
    'AttributeValue_SetRegionFlag',
    'AttributeValue_GetRegionFlag',
    'AttributeValue_SetGroupType',
    'AttributeValue_GetGroupType',
    'AttributeValue_SetGroupFlag',
    'AttributeValue_GetGroupFlag',
    'AttributeValue_SetBase',
    'AttributeValue_GetBase',
    'AttributeValue_SetMetricOccurrence',
    'AttributeValue_GetMetricOccurrence',
    'AttributeValue_SetMetricType',
    'AttributeValue_GetMetricType',
    'AttributeValue_SetMetricValueProperty',
    'AttributeValue_GetMetricValueProperty',
    'AttributeValue_SetMetricTiming',
    'AttributeValue_GetMetricTiming',
    'AttributeValue_SetMetricMode',
    'AttributeValue_GetMetricMode',
    'AttributeValue_SetMetricScope',
    'AttributeValue_GetMetricScope',
    'AttributeValue_SetRecorderKind',
    'AttributeValue_GetRecorderKind',
    'AttributeValue_SetParameterType',
    'AttributeValue_GetParameterType',
    'AttributeValue_SetCartPeriodicity',
    'AttributeValue_GetCartPeriodicity',
    'AttributeValue_SetInterruptGeneratorMode',
    'AttributeValue_GetInterruptGeneratorMode',
    'AttributeValue_SetMeasurementMode',
    'AttributeValue_GetMeasurementMode',
    'AttributeValue_SetCollectiveOp',
    'AttributeValue_GetCollectiveOp',
    'AttributeValue_SetRmaSyncType',
    'AttributeValue_GetRmaSyncType',
    'AttributeValue_SetRmaSyncLevel',
    'AttributeValue_GetRmaSyncLevel',
    'AttributeValue_SetLockType',
    'AttributeValue_GetLockType',
    'AttributeValue_SetRmaAtomicType',
    'AttributeValue_GetRmaAtomicType',
    'AttributeValue_SetIoParadigmClass',
    'AttributeValue_GetIoParadigmClass',
    'AttributeValue_SetIoParadigmFlag',
    'AttributeValue_GetIoParadigmFlag',
    'AttributeValue_SetIoParadigmProperty',
    'AttributeValue_GetIoParadigmProperty',
    'AttributeValue_SetIoHandleFlag',
    'AttributeValue_GetIoHandleFlag',
    'AttributeValue_SetIoAccessMode',
    'AttributeValue_GetIoAccessMode',
    'AttributeValue_SetIoCreationFlag',
    'AttributeValue_GetIoCreationFlag',
    'AttributeValue_SetIoStatusFlag',
    'AttributeValue_GetIoStatusFlag',
    'AttributeValue_SetIoSeekOption',
    'AttributeValue_GetIoSeekOption',
    'AttributeValue_SetIoOperationMode',
    'AttributeValue_GetIoOperationMode',
    'AttributeValue_SetIoOperationFlag',
    'AttributeValue_GetIoOperationFlag',
    'AttributeValue_SetCommFlag',
    'AttributeValue_GetCommFlag',
    'AttributeValue_SetRmaWinFlag',
    'AttributeValue_GetRmaWinFlag',
    'AttributeValue_SetCollectiveRoot',
    'AttributeValue_GetCollectiveRoot',
]



import ctypes

from .GeneralDefinitions import UNDEFINED_UINT32

MarkerRef = ctypes.c_uint8
UNDEFINED_MARKER = UNDEFINED_UINT32

MarkerSeverity = ctypes.c_uint8

SEVERITY_NONE = MarkerSeverity(0)
SEVERITY_LOW = MarkerSeverity(1)
SEVERITY_MEDIUM = MarkerSeverity(2)
SEVERITY_HIGH = MarkerSeverity(3)

MarkerScope = ctypes.c_uint8

MARKER_SCOPE_GLOBAL = MarkerScope(0)
MARKER_SCOPE_LOCATION = MarkerScope(1)
MARKER_SCOPE_LOCATION_GROUP = MarkerScope(2)
MARKER_SCOPE_SYSTEM_TREE_NODE = MarkerScope(3)
MARKER_SCOPE_GROUP = MarkerScope(4)
MARKER_SCOPE_COMM = MarkerScope(5)

__all__ = [
    'MarkerRef',
    'UNDEFINED_MARKER',
    'MarkerSeverity',
    'SEVERITY_NONE',
    'SEVERITY_LOW',
    'SEVERITY_MEDIUM',
    'SEVERITY_HIGH',
    'MarkerScope',
    'MARKER_SCOPE_GLOBAL',
    'MARKER_SCOPE_LOCATION',
    'MARKER_SCOPE_LOCATION_GROUP',
    'MARKER_SCOPE_SYSTEM_TREE_NODE',
    'MARKER_SCOPE_GROUP',
    'MARKER_SCOPE_COMM'
]



__copyright__ = "Copyright (c) 2015-2023, Technische Universitaet Dresden, Germany"
__license__ = "BSD"
from _otf2 import __version__

from . import writer
from . import reader
from .enums import *
from . import events
from . import definitions



from contextlib import closing
import time

import _otf2

from .definitions import Location, ClockProperties
from .definition_writer import DefinitionWriter
from .enums import *
from .error import TraceWriterError
from .event_writer import EventWriter
from .registry import DefinitionRegistry


# The contextlib.closing documentation actually suggests to name this thing open
# so we do this despite the shadowing
def open(*args, **kwargs):
    """
        Returns a :py:class:`otf2.reader.Reader` object usable within a :py:obj:`with`
        statement.

        :param str anchor_path: the path to the trace root directory
        :param str archive_name: the name of the trace archive (default: "traces")
    """

    return closing(Writer(*args, **kwargs))


def _default_pre_flush(user_data, file_type, flush_location, caller_data, final):
    return FlushType.FLUSH


def _use_param_doc_from(doc_func):
    def wrapper(func):
        doc = func.__doc__.format(params=doc_func.__doc__)
        func.__doc__ = doc
        return func
    return wrapper


class Writer(object):
    """
        This class is used to write a new trace file.

        .. Warning:: Parallel trace writing using collective callbacks is currently not supported.

        :param str anchor_path: the path to the trace root directory
        :param str archive_name: the name of the trace archive (default: 'traces')
        :param int timer_resolution: the resolution of the used timer (default: 1)
        :param otf2.registry.DefinitionRegistry definitions: used to share definitions with
            other :py:class:`otf2.writer.Writer` and :py:class:`otf2.reader.Reader` (default: None)
    """
    def __init__(self, archive_path,
                 archive_name="traces",
                 file_mode=FileMode.WRITE,
                 chunk_size_events=1024 * 1024, chunk_size_definitions=4 * 1024 * 1024,
                 file_substrate=FileSubstrate.POSIX, compression=Compression.NONE,
                 pre_flush=_default_pre_flush, post_flush=None, flush_data=None,
                 collective_callbacks=None, collective_data=None, global_comm_context=None,
                 local_comm_context=None, memory_callbacks=None, memory_data=None,
                 timer_resolution=None, definitions=None):

        self._file_mode = file_mode

        self._realtime_timestamp = None
        self._first_timestamp = None
        self._last_timestamp = None

        self._event_writers = dict()

        self._handle = _otf2.Archive_Open(archive_path, archive_name, file_mode,
                                          chunk_size_events, chunk_size_definitions,
                                          file_substrate, compression)

        # We MUST keep those flush callbacks, they cannot be GC'ed...
        self._flush_callbacks = _otf2.FlushCallbacks(pre_flush=pre_flush, post_flush=post_flush)
        _otf2.Archive_SetFlushCallbacks(self._handle, self._flush_callbacks, flush_data)

        if collective_callbacks is None:
            _otf2.Archive_SetSerialCollectiveCallbacks(self._handle)
        else:
            _otf2.Archive_SetCollectiveCallbacks(self._handle, collective_callbacks,
                                                 collective_data, global_comm_context,
                                                 local_comm_context)

        if memory_callbacks is not None:
            _otf2.Archive_SetMemoryCallbacks(self._handle, memory_callbacks, memory_data)

        _otf2.Archive_OpenDefFiles(self._handle)
        _otf2.Archive_OpenEvtFiles(self._handle)

        if definitions is None:
            definitions = DefinitionRegistry()

            # Reserve id 0 for empty string
            definitions.strings.create("")

        if timer_resolution is None:
            if definitions.clock_properties is None:
                definitions.clock_properties = ClockProperties(None, timer_resolution=1)
        else:
            if definitions.clock_properties is None:
                definitions.clock_properties = ClockProperties(None,
                                                               timer_resolution=timer_resolution)
            else:
                definitions.clock_properties.timer_resolution = timer_resolution

        self._global_definition_registry = definitions

    def _write_global_definitions(self):
        if not self.is_primary:
            return
        with DefinitionWriter(self) as global_def_writer:
            if self._first_timestamp is not None:
                duration = self._last_timestamp - self._first_timestamp
                global_offset = self._first_timestamp
                realtime_timestamp = int(self._realtime_timestamp * 1000000000)
            else:
                # No timestamps
                duration = 0
                global_offset = 0
                realtime_timestamp = Undefined.TIMESTAMP.value
            global_def_writer.write_clock_properties(self.timer_resolution,
                                                     global_offset, duration,
                                                     realtime_timestamp)
            self.definitions.write(global_def_writer)

    def close(self):
        if self._handle is None:
            return
        self._close_event_writers()
        _otf2.Archive_CloseEvtFiles(self._handle)
        _otf2.Archive_CloseDefFiles(self._handle)

        self._write_global_definitions()

        _otf2.Archive_Close(self._handle)
        self._handle = None

    # This is rather dangerous... Not good to have exceptions here
    def __del__(self):
        self.close()

    def _update_timestamps(self, timestamp):
        if self._first_timestamp is None:
            self._realtime_timestamp = time.time()
            self._first_timestamp = timestamp
            self._last_timestamp = timestamp
        else:
            self._first_timestamp = min(self._first_timestamp, timestamp)
            self._last_timestamp = max(self._last_timestamp, timestamp)

    def event_writer_from_location(self, location):
        """
            Returns an :py:class:`otf2.event_writer.EventWriter` for the given
            :py:class:`otf2.definitions.Location`.

            :param otf2.definitions.Location location: the location definition
            :return: :py:class:`otf2.event_writer.EventWriter`
        """
        if not isinstance(location, Location):
            raise TypeError("Wrong type for first argument. Expected a Location. Got {}."
                            .format(type(location)))
        if location._ref not in self._event_writers:
            self._event_writers[location._ref] = EventWriter(self, location)
        return self._event_writers[location._ref]

    @_use_param_doc_from(Location)
    def event_writer(self, *args, **kwargs):
        """
            Returns an :py:class:`otf2.event_writer.EventWriter` for the
            :py:class:`otf2.definitions.Location` equivalent to the given arguments.

{params}
:return: :py:class:`otf2.event_writer.EventWriter`
        """
        # I know, the last bit in the doc comment is pretty ugly, but it is necessary to remove the
        # indentation, otherwise Sphinx freaks out :(
        if len(args) > 0 and isinstance(args[0], Location):
            raise TypeError("Wrong type for first argument.")
        location = self.definitions.location(*args, **kwargs)
        return self.event_writer_from_location(location)

    @property
    def definitions(self):
        """
            Provides access to the global trace definitions.
            In the parallel case, all calls to the definitions must be collective.

            :return: :py:class:`otf2.registry.DefinitionRegistry`
        """
        return self._global_definition_registry

    @property
    def timer_resolution(self):
        """
            Returns the resolution of the used timer
        """
        return self.definitions.clock_properties.timer_resolution

    @property
    def handle(self):
        """
            Returns the handle to the underlaying OTF2 Archive.
        """
        if self._handle is None:
            raise TraceWriterError("Trying to access handle, but writer is already closed.")
        return self._handle

    def _close_event_writers(self):
        for writer in self._event_writers.values():
            writer.close()

    # setter/getter
    @property
    def file_mode(self):
        return self._file_mode

    @file_mode.setter
    def file_mode(self, new_file_mode):
        _otf2.Archive_SwitchFileMode(self._handle, new_file_mode)
        self._file_mode = new_file_mode

    @property
    def machine_name(self):
        return _otf2.Archive_GetMachineName(self._handle)

    @machine_name.setter
    def machine_name(self, machine_name):
        _otf2.Archive_SetMachineName(self._handle, machine_name)

    @property
    def description(self):
        return _otf2.Archive_GetDescription(self._handle)

    @description.setter
    def description(self, description):
        _otf2.Archive_SetDescription(self._handle, description)

    @property
    def creator(self):
        return _otf2.Archive_GetCreator(self._handle)

    @creator.setter
    def creator(self, creator):
        _otf2.Archive_SetCreator(self._handle, creator)

    # Read only
    @property
    def is_primary(self):
        return _otf2.Archive_IsPrimary(self._handle)

    # Read only, deprecated
    @property
    def is_master(self):
        return self.is_primary

    @property
    def version(self):
        return _otf2.Archive_GetVersion(self._handle)

    @property
    def chunk_size(self):
        return _otf2.Archive_GetChunkSize(self._handle)

    @property
    def file_substrate(self):
        return _otf2.Archive_GetFileSubstrate(self._handle)

    @property
    def compression(self):
        return _otf2.Archive_GetCompression(self._handle)

    @property
    def trace_id(self):
        return _otf2.Archive_GetTraceId(self._handle)



# Generated by OTF2 Template Engine

import _otf2

from . import events


def _generate_doc_from(event):
    def decorator(func):
        doc_str = "Writes a new :py:class:`otf2.events.{}` event to the corresponding location of \
                  the writer.\n".format(event.__name__)
        doc_str += "\n"

        for name, type, desc in event._fields:
            if type.endswith("Ref"):
                type = "otf2.definitions.{}".format(type[:-3])
            elif "int" in type:
                type = "int"
            elif type.startswith("dict"):
                pass
            elif type.startswith("array"):
                pass
            else:
                type = "otf2.{}".format(type)
            doc_str += ":param {type} {name}: {desc}\n".format(name=name, type=type, desc=desc)
        func.__doc__ = doc_str
        return func
    return decorator


class EventWriter(object):
    def __init__(self, archive, location):
        assert archive
        assert location
        self._archive = archive
        self._location = location

        # Not used yet, but to prevent false errors, when reading
        self._def_handle = _otf2.Archive_GetDefWriter(archive._handle, location._ref)

        self._handle = _otf2.Archive_GetEvtWriter(archive._handle, location._ref)

        location._number_of_events_written = 0

    def close(self):
        _otf2.Archive_CloseDefWriter(self._archive._handle, self._def_handle)
        _otf2.Archive_CloseEvtWriter(self._archive._handle, self._handle)

    def write(self, event):
        """
            Writes a given event to the corresponding location

            :param event: :py:class:`otf2.events._Event`
        """
        self._archive._update_timestamps(event.time)
        self._location._number_of_events_written += 1
        event._write(self)

    @property
    def handle(self):
        """
            Gives access to the underlaying OTF2 event writer
        """
        return self._handle

    def __call__(self, event):
        self.write(event)

    @_generate_doc_from(events.BufferFlush)
    def buffer_flush(self, time, *args, **kwargs):
        self.write(events.BufferFlush(time, *args, **kwargs))

    @_generate_doc_from(events.MeasurementOnOff)
    def measurement_on_off(self, time, *args, **kwargs):
        self.write(events.MeasurementOnOff(time, *args, **kwargs))

    @_generate_doc_from(events.Enter)
    def enter(self, time, *args, **kwargs):
        self.write(events.Enter(time, *args, **kwargs))

    @_generate_doc_from(events.Leave)
    def leave(self, time, *args, **kwargs):
        self.write(events.Leave(time, *args, **kwargs))

    @_generate_doc_from(events.MpiSend)
    def mpi_send(self, time, *args, **kwargs):
        self.write(events.MpiSend(time, *args, **kwargs))

    @_generate_doc_from(events.MpiIsend)
    def mpi_isend(self, time, *args, **kwargs):
        self.write(events.MpiIsend(time, *args, **kwargs))

    @_generate_doc_from(events.MpiIsendComplete)
    def mpi_isend_complete(self, time, *args, **kwargs):
        self.write(events.MpiIsendComplete(time, *args, **kwargs))

    @_generate_doc_from(events.MpiIrecvRequest)
    def mpi_irecv_request(self, time, *args, **kwargs):
        self.write(events.MpiIrecvRequest(time, *args, **kwargs))

    @_generate_doc_from(events.MpiRecv)
    def mpi_recv(self, time, *args, **kwargs):
        self.write(events.MpiRecv(time, *args, **kwargs))

    @_generate_doc_from(events.MpiIrecv)
    def mpi_irecv(self, time, *args, **kwargs):
        self.write(events.MpiIrecv(time, *args, **kwargs))

    @_generate_doc_from(events.MpiRequestTest)
    def mpi_request_test(self, time, *args, **kwargs):
        self.write(events.MpiRequestTest(time, *args, **kwargs))

    @_generate_doc_from(events.MpiRequestCancelled)
    def mpi_request_cancelled(self, time, *args, **kwargs):
        self.write(events.MpiRequestCancelled(time, *args, **kwargs))

    @_generate_doc_from(events.MpiCollectiveBegin)
    def mpi_collective_begin(self, time, *args, **kwargs):
        self.write(events.MpiCollectiveBegin(time, *args, **kwargs))

    @_generate_doc_from(events.MpiCollectiveEnd)
    def mpi_collective_end(self, time, *args, **kwargs):
        self.write(events.MpiCollectiveEnd(time, *args, **kwargs))

    @_generate_doc_from(events.OmpFork)
    def omp_fork(self, time, *args, **kwargs):
        self.write(events.OmpFork(time, *args, **kwargs))

    @_generate_doc_from(events.OmpJoin)
    def omp_join(self, time, *args, **kwargs):
        self.write(events.OmpJoin(time, *args, **kwargs))

    @_generate_doc_from(events.OmpAcquireLock)
    def omp_acquire_lock(self, time, *args, **kwargs):
        self.write(events.OmpAcquireLock(time, *args, **kwargs))

    @_generate_doc_from(events.OmpReleaseLock)
    def omp_release_lock(self, time, *args, **kwargs):
        self.write(events.OmpReleaseLock(time, *args, **kwargs))

    @_generate_doc_from(events.OmpTaskCreate)
    def omp_task_create(self, time, *args, **kwargs):
        self.write(events.OmpTaskCreate(time, *args, **kwargs))

    @_generate_doc_from(events.OmpTaskSwitch)
    def omp_task_switch(self, time, *args, **kwargs):
        self.write(events.OmpTaskSwitch(time, *args, **kwargs))

    @_generate_doc_from(events.OmpTaskComplete)
    def omp_task_complete(self, time, *args, **kwargs):
        self.write(events.OmpTaskComplete(time, *args, **kwargs))

    @_generate_doc_from(events.Metric)
    def metric(self, time, *args, **kwargs):
        self.write(events.Metric(time, *args, **kwargs))

    @_generate_doc_from(events.ParameterString)
    def parameter_string(self, time, *args, **kwargs):
        self.write(events.ParameterString(time, *args, **kwargs))

    @_generate_doc_from(events.ParameterInt)
    def parameter_int(self, time, *args, **kwargs):
        self.write(events.ParameterInt(time, *args, **kwargs))

    @_generate_doc_from(events.ParameterUnsignedInt)
    def parameter_unsigned_int(self, time, *args, **kwargs):
        self.write(events.ParameterUnsignedInt(time, *args, **kwargs))

    @_generate_doc_from(events.RmaWinCreate)
    def rma_win_create(self, time, *args, **kwargs):
        self.write(events.RmaWinCreate(time, *args, **kwargs))

    @_generate_doc_from(events.RmaWinDestroy)
    def rma_win_destroy(self, time, *args, **kwargs):
        self.write(events.RmaWinDestroy(time, *args, **kwargs))

    @_generate_doc_from(events.RmaCollectiveBegin)
    def rma_collective_begin(self, time, *args, **kwargs):
        self.write(events.RmaCollectiveBegin(time, *args, **kwargs))

    @_generate_doc_from(events.RmaCollectiveEnd)
    def rma_collective_end(self, time, *args, **kwargs):
        self.write(events.RmaCollectiveEnd(time, *args, **kwargs))

    @_generate_doc_from(events.RmaGroupSync)
    def rma_group_sync(self, time, *args, **kwargs):
        self.write(events.RmaGroupSync(time, *args, **kwargs))

    @_generate_doc_from(events.RmaRequestLock)
    def rma_request_lock(self, time, *args, **kwargs):
        self.write(events.RmaRequestLock(time, *args, **kwargs))

    @_generate_doc_from(events.RmaAcquireLock)
    def rma_acquire_lock(self, time, *args, **kwargs):
        self.write(events.RmaAcquireLock(time, *args, **kwargs))

    @_generate_doc_from(events.RmaTryLock)
    def rma_try_lock(self, time, *args, **kwargs):
        self.write(events.RmaTryLock(time, *args, **kwargs))

    @_generate_doc_from(events.RmaReleaseLock)
    def rma_release_lock(self, time, *args, **kwargs):
        self.write(events.RmaReleaseLock(time, *args, **kwargs))

    @_generate_doc_from(events.RmaSync)
    def rma_sync(self, time, *args, **kwargs):
        self.write(events.RmaSync(time, *args, **kwargs))

    @_generate_doc_from(events.RmaWaitChange)
    def rma_wait_change(self, time, *args, **kwargs):
        self.write(events.RmaWaitChange(time, *args, **kwargs))

    @_generate_doc_from(events.RmaPut)
    def rma_put(self, time, *args, **kwargs):
        self.write(events.RmaPut(time, *args, **kwargs))

    @_generate_doc_from(events.RmaGet)
    def rma_get(self, time, *args, **kwargs):
        self.write(events.RmaGet(time, *args, **kwargs))

    @_generate_doc_from(events.RmaAtomic)
    def rma_atomic(self, time, *args, **kwargs):
        self.write(events.RmaAtomic(time, *args, **kwargs))

    @_generate_doc_from(events.RmaOpCompleteBlocking)
    def rma_op_complete_blocking(self, time, *args, **kwargs):
        self.write(events.RmaOpCompleteBlocking(time, *args, **kwargs))

    @_generate_doc_from(events.RmaOpCompleteNonBlocking)
    def rma_op_complete_non_blocking(self, time, *args, **kwargs):
        self.write(events.RmaOpCompleteNonBlocking(time, *args, **kwargs))

    @_generate_doc_from(events.RmaOpTest)
    def rma_op_test(self, time, *args, **kwargs):
        self.write(events.RmaOpTest(time, *args, **kwargs))

    @_generate_doc_from(events.RmaOpCompleteRemote)
    def rma_op_complete_remote(self, time, *args, **kwargs):
        self.write(events.RmaOpCompleteRemote(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadFork)
    def thread_fork(self, time, *args, **kwargs):
        self.write(events.ThreadFork(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadJoin)
    def thread_join(self, time, *args, **kwargs):
        self.write(events.ThreadJoin(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadTeamBegin)
    def thread_team_begin(self, time, *args, **kwargs):
        self.write(events.ThreadTeamBegin(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadTeamEnd)
    def thread_team_end(self, time, *args, **kwargs):
        self.write(events.ThreadTeamEnd(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadAcquireLock)
    def thread_acquire_lock(self, time, *args, **kwargs):
        self.write(events.ThreadAcquireLock(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadReleaseLock)
    def thread_release_lock(self, time, *args, **kwargs):
        self.write(events.ThreadReleaseLock(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadTaskCreate)
    def thread_task_create(self, time, *args, **kwargs):
        self.write(events.ThreadTaskCreate(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadTaskSwitch)
    def thread_task_switch(self, time, *args, **kwargs):
        self.write(events.ThreadTaskSwitch(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadTaskComplete)
    def thread_task_complete(self, time, *args, **kwargs):
        self.write(events.ThreadTaskComplete(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadCreate)
    def thread_create(self, time, *args, **kwargs):
        self.write(events.ThreadCreate(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadBegin)
    def thread_begin(self, time, *args, **kwargs):
        self.write(events.ThreadBegin(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadWait)
    def thread_wait(self, time, *args, **kwargs):
        self.write(events.ThreadWait(time, *args, **kwargs))

    @_generate_doc_from(events.ThreadEnd)
    def thread_end(self, time, *args, **kwargs):
        self.write(events.ThreadEnd(time, *args, **kwargs))

    @_generate_doc_from(events.CallingContextEnter)
    def calling_context_enter(self, time, *args, **kwargs):
        self.write(events.CallingContextEnter(time, *args, **kwargs))

    @_generate_doc_from(events.CallingContextLeave)
    def calling_context_leave(self, time, *args, **kwargs):
        self.write(events.CallingContextLeave(time, *args, **kwargs))

    @_generate_doc_from(events.CallingContextSample)
    def calling_context_sample(self, time, *args, **kwargs):
        self.write(events.CallingContextSample(time, *args, **kwargs))

    @_generate_doc_from(events.IoCreateHandle)
    def io_create_handle(self, time, *args, **kwargs):
        self.write(events.IoCreateHandle(time, *args, **kwargs))

    @_generate_doc_from(events.IoDestroyHandle)
    def io_destroy_handle(self, time, *args, **kwargs):
        self.write(events.IoDestroyHandle(time, *args, **kwargs))

    @_generate_doc_from(events.IoDuplicateHandle)
    def io_duplicate_handle(self, time, *args, **kwargs):
        self.write(events.IoDuplicateHandle(time, *args, **kwargs))

    @_generate_doc_from(events.IoSeek)
    def io_seek(self, time, *args, **kwargs):
        self.write(events.IoSeek(time, *args, **kwargs))

    @_generate_doc_from(events.IoChangeStatusFlags)
    def io_change_status_flags(self, time, *args, **kwargs):
        self.write(events.IoChangeStatusFlags(time, *args, **kwargs))

    @_generate_doc_from(events.IoDeleteFile)
    def io_delete_file(self, time, *args, **kwargs):
        self.write(events.IoDeleteFile(time, *args, **kwargs))

    @_generate_doc_from(events.IoOperationBegin)
    def io_operation_begin(self, time, *args, **kwargs):
        self.write(events.IoOperationBegin(time, *args, **kwargs))

    @_generate_doc_from(events.IoOperationTest)
    def io_operation_test(self, time, *args, **kwargs):
        self.write(events.IoOperationTest(time, *args, **kwargs))

    @_generate_doc_from(events.IoOperationIssued)
    def io_operation_issued(self, time, *args, **kwargs):
        self.write(events.IoOperationIssued(time, *args, **kwargs))

    @_generate_doc_from(events.IoOperationComplete)
    def io_operation_complete(self, time, *args, **kwargs):
        self.write(events.IoOperationComplete(time, *args, **kwargs))

    @_generate_doc_from(events.IoOperationCancelled)
    def io_operation_cancelled(self, time, *args, **kwargs):
        self.write(events.IoOperationCancelled(time, *args, **kwargs))

    @_generate_doc_from(events.IoAcquireLock)
    def io_acquire_lock(self, time, *args, **kwargs):
        self.write(events.IoAcquireLock(time, *args, **kwargs))

    @_generate_doc_from(events.IoReleaseLock)
    def io_release_lock(self, time, *args, **kwargs):
        self.write(events.IoReleaseLock(time, *args, **kwargs))

    @_generate_doc_from(events.IoTryLock)
    def io_try_lock(self, time, *args, **kwargs):
        self.write(events.IoTryLock(time, *args, **kwargs))

    @_generate_doc_from(events.ProgramBegin)
    def program_begin(self, time, *args, **kwargs):
        self.write(events.ProgramBegin(time, *args, **kwargs))

    @_generate_doc_from(events.ProgramEnd)
    def program_end(self, time, *args, **kwargs):
        self.write(events.ProgramEnd(time, *args, **kwargs))

    @_generate_doc_from(events.NonBlockingCollectiveRequest)
    def non_blocking_collective_request(self, time, *args, **kwargs):
        self.write(events.NonBlockingCollectiveRequest(time, *args, **kwargs))

    @_generate_doc_from(events.NonBlockingCollectiveComplete)
    def non_blocking_collective_complete(self, time, *args, **kwargs):
        self.write(events.NonBlockingCollectiveComplete(time, *args, **kwargs))

    @_generate_doc_from(events.CommCreate)
    def comm_create(self, time, *args, **kwargs):
        self.write(events.CommCreate(time, *args, **kwargs))

    @_generate_doc_from(events.CommDestroy)
    def comm_destroy(self, time, *args, **kwargs):
        self.write(events.CommDestroy(time, *args, **kwargs))

    def _buffer_flush(self, attribute_list, time, stop_time):
        _otf2.EvtWriter_BufferFlush(self._handle, attribute_list, time, stop_time)

    def _measurement_on_off(self, attribute_list, time, measurement_mode):
        _otf2.EvtWriter_MeasurementOnOff(self._handle, attribute_list, time, measurement_mode)

    def _enter(self, attribute_list, time, region):
        _otf2.EvtWriter_Enter(self._handle, attribute_list, time, region)

    def _leave(self, attribute_list, time, region):
        _otf2.EvtWriter_Leave(self._handle, attribute_list, time, region)

    def _mpi_send(self, attribute_list, time, receiver, communicator, msg_tag, msg_length):
        _otf2.EvtWriter_MpiSend(self._handle, attribute_list, time, receiver, communicator, msg_tag, msg_length)

    def _mpi_isend(self, attribute_list, time, receiver, communicator, msg_tag, msg_length, request_id):
        _otf2.EvtWriter_MpiIsend(self._handle, attribute_list, time, receiver, communicator, msg_tag, msg_length, request_id)

    def _mpi_isend_complete(self, attribute_list, time, request_id):
        _otf2.EvtWriter_MpiIsendComplete(self._handle, attribute_list, time, request_id)

    def _mpi_irecv_request(self, attribute_list, time, request_id):
        _otf2.EvtWriter_MpiIrecvRequest(self._handle, attribute_list, time, request_id)

    def _mpi_recv(self, attribute_list, time, sender, communicator, msg_tag, msg_length):
        _otf2.EvtWriter_MpiRecv(self._handle, attribute_list, time, sender, communicator, msg_tag, msg_length)

    def _mpi_irecv(self, attribute_list, time, sender, communicator, msg_tag, msg_length, request_id):
        _otf2.EvtWriter_MpiIrecv(self._handle, attribute_list, time, sender, communicator, msg_tag, msg_length, request_id)

    def _mpi_request_test(self, attribute_list, time, request_id):
        _otf2.EvtWriter_MpiRequestTest(self._handle, attribute_list, time, request_id)

    def _mpi_request_cancelled(self, attribute_list, time, request_id):
        _otf2.EvtWriter_MpiRequestCancelled(self._handle, attribute_list, time, request_id)

    def _mpi_collective_begin(self, attribute_list, time):
        _otf2.EvtWriter_MpiCollectiveBegin(self._handle, attribute_list, time)

    def _mpi_collective_end(self, attribute_list, time, collective_op, communicator, root, size_sent, size_received):
        _otf2.EvtWriter_MpiCollectiveEnd(self._handle, attribute_list, time, collective_op, communicator, root, size_sent, size_received)

    def _omp_fork(self, attribute_list, time, number_of_requested_threads):
        _otf2.EvtWriter_OmpFork(self._handle, attribute_list, time, number_of_requested_threads)

    def _omp_join(self, attribute_list, time):
        _otf2.EvtWriter_OmpJoin(self._handle, attribute_list, time)

    def _omp_acquire_lock(self, attribute_list, time, lock_id, acquisition_order):
        _otf2.EvtWriter_OmpAcquireLock(self._handle, attribute_list, time, lock_id, acquisition_order)

    def _omp_release_lock(self, attribute_list, time, lock_id, acquisition_order):
        _otf2.EvtWriter_OmpReleaseLock(self._handle, attribute_list, time, lock_id, acquisition_order)

    def _omp_task_create(self, attribute_list, time, task_id):
        _otf2.EvtWriter_OmpTaskCreate(self._handle, attribute_list, time, task_id)

    def _omp_task_switch(self, attribute_list, time, task_id):
        _otf2.EvtWriter_OmpTaskSwitch(self._handle, attribute_list, time, task_id)

    def _omp_task_complete(self, attribute_list, time, task_id):
        _otf2.EvtWriter_OmpTaskComplete(self._handle, attribute_list, time, task_id)

    def _metric(self, attribute_list, time, metric, type_ids, metric_values):
        _otf2.EvtWriter_Metric(self._handle, attribute_list, time, metric, type_ids, metric_values)

    def _parameter_string(self, attribute_list, time, parameter, string):
        _otf2.EvtWriter_ParameterString(self._handle, attribute_list, time, parameter, string)

    def _parameter_int(self, attribute_list, time, parameter, value):
        _otf2.EvtWriter_ParameterInt(self._handle, attribute_list, time, parameter, value)

    def _parameter_unsigned_int(self, attribute_list, time, parameter, value):
        _otf2.EvtWriter_ParameterUnsignedInt(self._handle, attribute_list, time, parameter, value)

    def _rma_win_create(self, attribute_list, time, win):
        _otf2.EvtWriter_RmaWinCreate(self._handle, attribute_list, time, win)

    def _rma_win_destroy(self, attribute_list, time, win):
        _otf2.EvtWriter_RmaWinDestroy(self._handle, attribute_list, time, win)

    def _rma_collective_begin(self, attribute_list, time):
        _otf2.EvtWriter_RmaCollectiveBegin(self._handle, attribute_list, time)

    def _rma_collective_end(self, attribute_list, time, collective_op, sync_level, win, root, bytes_sent, bytes_received):
        _otf2.EvtWriter_RmaCollectiveEnd(self._handle, attribute_list, time, collective_op, sync_level, win, root, bytes_sent, bytes_received)

    def _rma_group_sync(self, attribute_list, time, sync_level, win, group):
        _otf2.EvtWriter_RmaGroupSync(self._handle, attribute_list, time, sync_level, win, group)

    def _rma_request_lock(self, attribute_list, time, win, remote, lock_id, lock_type):
        _otf2.EvtWriter_RmaRequestLock(self._handle, attribute_list, time, win, remote, lock_id, lock_type)

    def _rma_acquire_lock(self, attribute_list, time, win, remote, lock_id, lock_type):
        _otf2.EvtWriter_RmaAcquireLock(self._handle, attribute_list, time, win, remote, lock_id, lock_type)

    def _rma_try_lock(self, attribute_list, time, win, remote, lock_id, lock_type):
        _otf2.EvtWriter_RmaTryLock(self._handle, attribute_list, time, win, remote, lock_id, lock_type)

    def _rma_release_lock(self, attribute_list, time, win, remote, lock_id):
        _otf2.EvtWriter_RmaReleaseLock(self._handle, attribute_list, time, win, remote, lock_id)

    def _rma_sync(self, attribute_list, time, win, remote, sync_type):
        _otf2.EvtWriter_RmaSync(self._handle, attribute_list, time, win, remote, sync_type)

    def _rma_wait_change(self, attribute_list, time, win):
        _otf2.EvtWriter_RmaWaitChange(self._handle, attribute_list, time, win)

    def _rma_put(self, attribute_list, time, win, remote, bytes, matching_id):
        _otf2.EvtWriter_RmaPut(self._handle, attribute_list, time, win, remote, bytes, matching_id)

    def _rma_get(self, attribute_list, time, win, remote, bytes, matching_id):
        _otf2.EvtWriter_RmaGet(self._handle, attribute_list, time, win, remote, bytes, matching_id)

    def _rma_atomic(self, attribute_list, time, win, remote, type, bytes_sent, bytes_received, matching_id):
        _otf2.EvtWriter_RmaAtomic(self._handle, attribute_list, time, win, remote, type, bytes_sent, bytes_received, matching_id)

    def _rma_op_complete_blocking(self, attribute_list, time, win, matching_id):
        _otf2.EvtWriter_RmaOpCompleteBlocking(self._handle, attribute_list, time, win, matching_id)

    def _rma_op_complete_non_blocking(self, attribute_list, time, win, matching_id):
        _otf2.EvtWriter_RmaOpCompleteNonBlocking(self._handle, attribute_list, time, win, matching_id)

    def _rma_op_test(self, attribute_list, time, win, matching_id):
        _otf2.EvtWriter_RmaOpTest(self._handle, attribute_list, time, win, matching_id)

    def _rma_op_complete_remote(self, attribute_list, time, win, matching_id):
        _otf2.EvtWriter_RmaOpCompleteRemote(self._handle, attribute_list, time, win, matching_id)

    def _thread_fork(self, attribute_list, time, model, number_of_requested_threads):
        _otf2.EvtWriter_ThreadFork(self._handle, attribute_list, time, model, number_of_requested_threads)

    def _thread_join(self, attribute_list, time, model):
        _otf2.EvtWriter_ThreadJoin(self._handle, attribute_list, time, model)

    def _thread_team_begin(self, attribute_list, time, thread_team):
        _otf2.EvtWriter_ThreadTeamBegin(self._handle, attribute_list, time, thread_team)

    def _thread_team_end(self, attribute_list, time, thread_team):
        _otf2.EvtWriter_ThreadTeamEnd(self._handle, attribute_list, time, thread_team)

    def _thread_acquire_lock(self, attribute_list, time, model, lock_id, acquisition_order):
        _otf2.EvtWriter_ThreadAcquireLock(self._handle, attribute_list, time, model, lock_id, acquisition_order)

    def _thread_release_lock(self, attribute_list, time, model, lock_id, acquisition_order):
        _otf2.EvtWriter_ThreadReleaseLock(self._handle, attribute_list, time, model, lock_id, acquisition_order)

    def _thread_task_create(self, attribute_list, time, thread_team, creating_thread, generation_number):
        _otf2.EvtWriter_ThreadTaskCreate(self._handle, attribute_list, time, thread_team, creating_thread, generation_number)

    def _thread_task_switch(self, attribute_list, time, thread_team, creating_thread, generation_number):
        _otf2.EvtWriter_ThreadTaskSwitch(self._handle, attribute_list, time, thread_team, creating_thread, generation_number)

    def _thread_task_complete(self, attribute_list, time, thread_team, creating_thread, generation_number):
        _otf2.EvtWriter_ThreadTaskComplete(self._handle, attribute_list, time, thread_team, creating_thread, generation_number)

    def _thread_create(self, attribute_list, time, thread_contingent, sequence_count):
        _otf2.EvtWriter_ThreadCreate(self._handle, attribute_list, time, thread_contingent, sequence_count)

    def _thread_begin(self, attribute_list, time, thread_contingent, sequence_count):
        _otf2.EvtWriter_ThreadBegin(self._handle, attribute_list, time, thread_contingent, sequence_count)

    def _thread_wait(self, attribute_list, time, thread_contingent, sequence_count):
        _otf2.EvtWriter_ThreadWait(self._handle, attribute_list, time, thread_contingent, sequence_count)

    def _thread_end(self, attribute_list, time, thread_contingent, sequence_count):
        _otf2.EvtWriter_ThreadEnd(self._handle, attribute_list, time, thread_contingent, sequence_count)

    def _calling_context_enter(self, attribute_list, time, calling_context, unwind_distance):
        _otf2.EvtWriter_CallingContextEnter(self._handle, attribute_list, time, calling_context, unwind_distance)

    def _calling_context_leave(self, attribute_list, time, calling_context):
        _otf2.EvtWriter_CallingContextLeave(self._handle, attribute_list, time, calling_context)

    def _calling_context_sample(self, attribute_list, time, calling_context, unwind_distance, interrupt_generator):
        _otf2.EvtWriter_CallingContextSample(self._handle, attribute_list, time, calling_context, unwind_distance, interrupt_generator)

    def _io_create_handle(self, attribute_list, time, handle, mode, creation_flags, status_flags):
        _otf2.EvtWriter_IoCreateHandle(self._handle, attribute_list, time, handle, mode, creation_flags, status_flags)

    def _io_destroy_handle(self, attribute_list, time, handle):
        _otf2.EvtWriter_IoDestroyHandle(self._handle, attribute_list, time, handle)

    def _io_duplicate_handle(self, attribute_list, time, old_handle, new_handle, status_flags):
        _otf2.EvtWriter_IoDuplicateHandle(self._handle, attribute_list, time, old_handle, new_handle, status_flags)

    def _io_seek(self, attribute_list, time, handle, offset_request, whence, offset_result):
        _otf2.EvtWriter_IoSeek(self._handle, attribute_list, time, handle, offset_request, whence, offset_result)

    def _io_change_status_flags(self, attribute_list, time, handle, status_flags):
        _otf2.EvtWriter_IoChangeStatusFlags(self._handle, attribute_list, time, handle, status_flags)

    def _io_delete_file(self, attribute_list, time, io_paradigm, file):
        _otf2.EvtWriter_IoDeleteFile(self._handle, attribute_list, time, io_paradigm, file)

    def _io_operation_begin(self, attribute_list, time, handle, mode, operation_flags, bytes_request, matching_id):
        _otf2.EvtWriter_IoOperationBegin(self._handle, attribute_list, time, handle, mode, operation_flags, bytes_request, matching_id)

    def _io_operation_test(self, attribute_list, time, handle, matching_id):
        _otf2.EvtWriter_IoOperationTest(self._handle, attribute_list, time, handle, matching_id)

    def _io_operation_issued(self, attribute_list, time, handle, matching_id):
        _otf2.EvtWriter_IoOperationIssued(self._handle, attribute_list, time, handle, matching_id)

    def _io_operation_complete(self, attribute_list, time, handle, bytes_result, matching_id):
        _otf2.EvtWriter_IoOperationComplete(self._handle, attribute_list, time, handle, bytes_result, matching_id)

    def _io_operation_cancelled(self, attribute_list, time, handle, matching_id):
        _otf2.EvtWriter_IoOperationCancelled(self._handle, attribute_list, time, handle, matching_id)

    def _io_acquire_lock(self, attribute_list, time, handle, lock_type):
        _otf2.EvtWriter_IoAcquireLock(self._handle, attribute_list, time, handle, lock_type)

    def _io_release_lock(self, attribute_list, time, handle, lock_type):
        _otf2.EvtWriter_IoReleaseLock(self._handle, attribute_list, time, handle, lock_type)

    def _io_try_lock(self, attribute_list, time, handle, lock_type):
        _otf2.EvtWriter_IoTryLock(self._handle, attribute_list, time, handle, lock_type)

    def _program_begin(self, attribute_list, time, program_name, program_arguments):
        _otf2.EvtWriter_ProgramBegin(self._handle, attribute_list, time, program_name, program_arguments)

    def _program_end(self, attribute_list, time, exit_status):
        _otf2.EvtWriter_ProgramEnd(self._handle, attribute_list, time, exit_status)

    def _non_blocking_collective_request(self, attribute_list, time, request_id):
        _otf2.EvtWriter_NonBlockingCollectiveRequest(self._handle, attribute_list, time, request_id)

    def _non_blocking_collective_complete(self, attribute_list, time, collective_op, communicator, root, size_sent, size_received, request_id):
        _otf2.EvtWriter_NonBlockingCollectiveComplete(self._handle, attribute_list, time, collective_op, communicator, root, size_sent, size_received, request_id)

    def _comm_create(self, attribute_list, time, communicator):
        _otf2.EvtWriter_CommCreate(self._handle, attribute_list, time, communicator)

    def _comm_destroy(self, attribute_list, time, communicator):
        _otf2.EvtWriter_CommDestroy(self._handle, attribute_list, time, communicator)
